package com.zeroturnaround.javarebel.guice;

import org.zeroturnaround.javarebel.ClassResourceSource;
import org.zeroturnaround.javarebel.IntegrationFactory;
import org.zeroturnaround.javarebel.LoggerFactory;
import org.zeroturnaround.javarebel.Plugin;
import org.zeroturnaround.javarebel.ReloaderFactory;

import com.google.inject.GuiceComponentReloader;
import com.zeroturnaround.javarebel.guice.cbp.InjectorImplCBP;
import com.zeroturnaround.javarebel.guice.cbp.ScopesCBP;

public class GuicePlugin implements Plugin {

	public void preinit() {
	    String property = System.getProperty("rebel.guice_plugin");
	    if (property != null && property.equalsIgnoreCase("false"))
	      return;
	    
	    LoggerFactory.getInstance().echo("---------------------------------------------------------------------------------------");
	    LoggerFactory.getInstance().echo("JavaRebel: The Guice Plugin is now enabled.");
	    LoggerFactory.getInstance().echo("JavaRebel: It supports adding new components and injecting new implicit dependencies ");
	    LoggerFactory.getInstance().echo("JavaRebel: Singletons are reconfigured after their class changes.");
	    LoggerFactory.getInstance().echo("JavaRebel: If you have any troubles add -Drebel.guice_plugin=false to disable it.");
	    LoggerFactory.getInstance().echo("---------------------------------------------------------------------------------------");
		
		ClassLoader cl = getClass().getClassLoader();
		
		ReloaderFactory
		.getInstance()
		.addClassReloadListener(
				new GuiceComponentReloader());
		
		IntegrationFactory
		.getInstance()
		.addIntegrationProcessor(
				cl, 
				"com.google.inject.InjectorImpl", 
				new InjectorImplCBP());
		
		IntegrationFactory
		.getInstance()
		.addIntegrationProcessor(
				cl, 
				"com.google.inject.Scopes", 
				new ScopesCBP());		
	}
	
	public boolean checkDependencies(ClassLoader cl, ClassResourceSource crs) {
		return crs.getClassResource("com.google.inject.Injector") != null;
	}
}
